//! This module defines the 2x2, 3x3, and 4x4 Matrix structures.
//use std::ops::{Add, Sub, Mul, Div, Rem, Neg};

use crate::number::Number;
use crate::vector::{Vector, Vector2, Vector3, Vector4};



/// A trait that defines the minimum set of
/// functions a [Matrix][1] must implement.
///
/// [1]: https://en.wikipedia.org/
pub trait Matrix<T, VT> : Sized
   where T: Number,
         VT: Vector<T>
{
   /// Create a Matrix with the given value set
   /// for all the diagonal values.
   fn from_value(value: T) -> Self;

   /// Create an Identity matrix. It will have 1 set
   /// for all the diagonal values.
   fn identity() -> Self
   {
      Self::from_value(T::one())
   }
}


macro_rules! define_matrix
{
   ($structName: ident <$T: ident>, $fieldType: ty {$($fieldName: ident),+}) =>
   {
      pub struct $structName<T>
      {
         $(pub $fieldName: $fieldType),+
      }
   }
}


define_matrix!(Matrix2<T>, Vector2<T> {x, y});
define_matrix!(Matrix3<T>, Vector3<T> {x, y, z});
define_matrix!(Matrix4<T>, Vector4<T> {x, y, z, w});
